// COLOR.H : some common (inline-)functions for colors...

// Copyright (C) 1999 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef COLOR_H
#define COLOR_H

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "typedef.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

// these should be done generic, to give any desired color range...
// these should be done generic, to give any desired color range...
// these should be done generic, to give any desired color range...

void GetRGRange1(fGL, fGL *);		// red -> yellow -> green

void GetRGRange2(fGL, fGL *);		// white -> red -> green -> white
void GetRGBRange2(fGL, fGL *);		// white -> red -> green -> blue -> white

void GetRedColor(fGL, fGL *);
void GetGreenColor(fGL, fGL *);
void GetBlueColor(fGL, fGL *);

fGL GetDistance(fGL *, void *, fGL *);
fGL GetUnity(fGL *, void *, fGL *);

////////////////////////////////////////////////////////////////////////////////////////////////////

inline void GetRGRange1(fGL value1, fGL * rgb)
{
	rgb[2] = 0.0;
	if (value1 < 0.5)
	{
		if (value1 < 0.0) value1 = 0.0;
		fGL value2 = 2.0 * value1;
		
		rgb[0] = 1.0;
		rgb[1] = value2;
	}
	else
	{
		if (value1 > 1.0) value1 = 1.0;
		fGL value2 = 2.0 * (value1 - 0.5);
		
		rgb[0] = 1.0 - value2;
		rgb[1] = 1.0;
	}
}

inline void GetRGRange2(fGL value1, fGL * rgb)
{
	fGL value2 = fabs(value1);
	if (value2 < 1.0)
	{
		rgb[2] = 0.0;
		if (value1 < 0.0)
		{
			rgb[0] = 0.0;
			rgb[1] = value2;
		}
		else
		{
			rgb[0] = value2;
			rgb[1] = 0.0;
		}
	}
	else
	{
		value2 = 1.0 - 1.0 / value2;
		if (value1 < 0.0)
		{
			rgb[0] = value2;
			rgb[1] = 1.0;
			rgb[2] = value2;
		}
		else
		{
			rgb[0] = 1.0;
			rgb[1] = value2;
			rgb[2] = value2;
		}
	}
}

inline void GetRGBRange2(fGL value1, fGL * rgb)
{
	fGL value2 = fabs(value1);
	if (value2 < 1.0)
	{
		rgb[1] = 1.0 - value2;
		if (value1 < 0.0)
		{
			rgb[0] = 0.0;
			rgb[2] = value2;
		}
		else
		{
			rgb[0] = value2;
			rgb[2] = 0.0;
		}
	}
	else
	{
		value2 = 1.0 - 1.0 / value2;
		if (value1 < 0.0)
		{
			rgb[0] = value2;
			rgb[1] = value2;
			rgb[2] = 1.0;
		}
		else
		{
			rgb[0] = 1.0;
			rgb[1] = value2;
			rgb[2] = value2;
		}
	}
}

inline void GetRedColor(fGL p1, fGL * p2)
{
	p2[0] = p1;
	p2[1] = p2[2] = 0.0;
}

inline void GetGreenColor(fGL p1, fGL * p2)
{
	p2[1] = p1;
	p2[0] = p2[2] = 0.0;
}

inline void GetBlueColor(fGL p1, fGL * p2)
{
	p2[2] = p1;
	p2[0] = p2[1] = 0.0;
}

inline fGL GetDistance(fGL * p1, void * p2, fGL *)
{
	fGL sum = 0.0;
	fGL * crd = (fGL *) p2;
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		fGL tmp = p1[n1] - crd[n1];
		sum += tmp * tmp;
	}
	
	return sqrt(sum);
}

inline fGL GetUnity(fGL *, void *, fGL *)
{
	return 1.0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// COLOR_H

// eof
