// CONJGRAD.H : a general-purpose conjugate-gradient optimizer class.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef CONJGRAD_H
#define CONJGRAD_H

class conjugate_gradient;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "define.h"
#include "typedef.h"

#include BEGIN_INCLUDE_STL	// this is compiler-dependent...
#include <vector>
#include END_INCLUDE_STL	// this is compiler-dependent...

////////////////////////////////////////////////////////////////////////////////////////////////////

struct cgvar
{
	f32 * ref1;	// ref to variable
	f32 * ref2;	// ref to derivative
	
	f32 data1;
	f32 data2;
};

class conjugate_gradient
{
	public:
	
	enum ls_mode			// line-search modes
	{
		Newton2An = 0,		// f'/f''-newton half numerical
		Newton2Num = 1,		// f'/f''-newton full numerical
		Newton1Num = 2,		// f/f'-newton full numerical
		SimpleTest = 3		// a simple test method
	};
	
	protected:
	
	i32s step; i32s reset;
	
	f32 newip; f32 oldip; f32 beta;
	f32 defstp; ls_mode mode;
	
	public:
	
	vector<cgvar> cgvar_vector;	// why public ???
	f32 optval; f32 optstp;
	f32 delta;
	
	public:
	
	conjugate_gradient(ls_mode, f32, i32s);
	virtual ~conjugate_gradient(void);
	
	void AddVar(f32 *, f32 *);
	
	bool TakeCGStep(void);
	void InitLineSearch(f32);
	
	virtual f32 GetValue(void) = 0;
	virtual f32 GetGradient(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// CONJGRAD_H

// eof
