// MM1ENG.H : calculations for "all atoms"-models (molecular mechanics).

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MM1ENG_H
#define MM1ENG_H

struct mm1_bt1_data;
struct mm1_bt2_data;

class mm1_eng;
class mm1_eng_pbc;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "engine.h"
#include "mm1mdl.h"

#include BEGIN_INCLUDE_STL	// this is compiler-dependent...
#include <vector>
#include END_INCLUDE_STL	// this is compiler-dependent...

////////////////////////////////////////////////////////////////////////////////////////////////////

struct mm1_bt1_data
{
	f32 len;
	f32 dlen[2][3];
};

struct mm1_bt2_data
{
	f32 csa;
	f32 dcsa[3][3];
};

void CopyCRD(mm1_mdl * p1, mm1_eng * p2, i32u p3);
void CopyCRD(mm1_eng * p1, mm1_mdl * p2, i32u p3);

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_eng : public bs_eng
{
	public:
	
	mm1_mdl * mdl;
	i32s np; i32s pid;
	
	i32s natm;
	
	f32_a3 * crd;
	f32_a3 * grad;
	
	f32 energy;
	f32 energy_bt1;
	f32 energy_bt2;
	f32 energy_bt3;
	f32 energy_nbt1;
	
	mm1_atom ** index;
	
	vector<i32s> cr1; i32s * range_cr1;
	vector<i32s> cr2; i32s * range_cr2;
	
	public:
	
	mm1_eng(mm1_mdl &, i32s, i32s);
	virtual ~mm1_eng(void);
	
	void Compute(i32s);		// virtual
	void Check(i32s);		// virtual
	
	private:
	
	void SearchCR1a(iter_mm1al);
	void SearchCR1b(iter_mm1cl, mm1_atom *);
	void SearchCR2(iter_mm1cl, mm1_atom *);
	
	protected:
	
	virtual void ComputeBT1(i32s) = 0;
	virtual void ComputeBT2(i32s) = 0;
	virtual void ComputeBT3(i32s) = 0;
	
	virtual void ComputeNBT1(i32s) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_eng_pbc : virtual public mm1_eng
{
	protected:
	
	i32s nmol;
	i32s * mrange;
	
	public:
	
	mm1_eng_pbc(mm1_mdl &, i32s, i32s);
	virtual ~mm1_eng_pbc(void);
	
	void CheckLocations(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// MM1ENG_H

// eof
