// MM1TAB1.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mm1tab1.h"

#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

////////////////////////////////////////////////////////////////////////////////////////////////////

exp1_tables * exp1_tables::instance = NULL;
singleton_cleaner<exp1_tables> exp1_cleaner;

exp1_tables::exp1_tables(void)
{
	const char * path = MM1_UNSTABLE_PATH;
	ostream * ostr = & cout;
	
	ifstream file;
	file.unsetf(ios::dec | ios::oct | ios::hex);
	char buffer[256];
	
	ostrstream atfn(buffer, sizeof(buffer));
	atfn << path << "atomtypes.txt" << ends; file.open(buffer, ios::in);
	if (file.good())
	{
		if (ostr != NULL) (* ostr) << "reading file \"" << buffer << "\": ";
		while (file.peek() != 'e')
		{
			if (file.peek() == '0')
			{
				exp1_at newat;
				file >> newat.atomtype;
				
				while (file.peek() != '(') file.get();
				newat.typerule = new mm1_typerule(& file, ostr);
				for (i32u n1 = 0;n1 < sizeof(buffer);n1++) buffer[n1] = 0;
				
				while (file.get() != '\"');
				file.getline(buffer, sizeof(buffer), '\"');
				
				newat.description = new char[strlen(buffer) + 1];
				strcpy(newat.description, buffer);
				at_vector.push_back(newat);
			}
			
			file.getline(buffer, sizeof(buffer));
		}
	} else if (ostr != NULL) (* ostr) << "could not open file " << buffer << endl;
	if (ostr != NULL) (* ostr) << "found " << at_vector.size() << " atomtypes." << endl;
	file.close();
	
	ostrstream bsfn(buffer, sizeof(buffer));
	bsfn << path << "parameters1.txt" << ends; file.open(buffer, ios::in);
	if (file.good())
	{
		if (ostr != NULL) (* ostr) << "reading file \"" << buffer << "\": ";
		while (file.peek() != 'e')
		{
			if (file.peek() == '0')
			{
				exp1_bs tmp; char bt[16];
				file >> tmp.atmtp[0] >> tmp.atmtp[1];
				file >> bt >> tmp.param[0] >> tmp.param[1];
				
				tmp.bndtp = bondtype(bt[0]);
				bs_vector.push_back(tmp);
			}
			
			file.getline(buffer, sizeof(buffer));
		}
	} else if (ostr != NULL) (* ostr) << "could not open file " << buffer << endl;
	if (ostr != NULL) (* ostr) << "found " << bs_vector.size() << " bs-terms." << endl;
	file.close();
	
	ostrstream abfn(buffer, sizeof(buffer));
	abfn << path << "parameters2.txt" << ends; file.open(buffer, ios::in);
	if (file.good())
	{
		if (ostr != NULL) (* ostr) << "reading file \"" << buffer << "\": ";
		while (file.peek() != 'e')
		{
			if (file.peek() == '0')
			{
				exp1_ab tmp; char bt[16];
				file >> tmp.atmtp[0] >> tmp.atmtp[1] >> tmp.atmtp[2];
				file >> bt >> tmp.param[0] >> tmp.param[1];
				
				for (i32s n1 = 0;n1 < 2;n1++) tmp.bndtp[n1] = bondtype(bt[n1]);
				ab_vector.push_back(tmp);
			}
			
			file.getline(buffer, sizeof(buffer));
		}
	} else if (ostr != NULL) (* ostr) << "could not open file " << buffer << endl;
	if (ostr != NULL) (* ostr) << "found " << ab_vector.size() << " ab-terms." << endl;
	file.close();
	
	ostrstream trfn(buffer, sizeof(buffer));
	trfn << path << "parameters3.txt" << ends; file.open(buffer, ios::in);
	if (file.good())
	{
		if (ostr != NULL) (* ostr) << "reading file \"" << buffer << "\": ";
		while (file.peek() != 'e')
		{
			if (file.peek() == '0')
			{
				exp1_tr tmp; char bt[16];
				file >> tmp.atmtp[0] >> tmp.atmtp[1] >> tmp.atmtp[2] >> tmp.atmtp[3];
				file >> bt >> tmp.k >> tmp.s;
				
				for (i32s n1 = 0;n1 < 3;n1++) tmp.bndtp[n1] = bondtype(bt[n1]);
				tr_vector.push_back(tmp);
			}
			
			file.getline(buffer, sizeof(buffer));
		}
	} else if (ostr != NULL) (*ostr) << "could not open file " << buffer << endl;
	if (ostr != NULL) (* ostr) << "found " << tr_vector.size() << " tr-terms." << endl;
	file.close();
	
	ostrstream ljfn(buffer, sizeof(buffer));
	ljfn << path << "parameters4.txt" << ends; file.open(buffer, ios::in);
	if (file.good())
	{
		if (ostr != NULL) (* ostr) << "reading file \"" << buffer << "\": ";
		while (file.peek() != 'e')
		{
			if (file.peek() == '0')
			{
				exp1_lj tmp;
				file >> tmp.type >> tmp.r >> tmp.k;
				lj_vector.push_back(tmp);
			}
			
			file.getline(buffer, sizeof(buffer));
		}
	} else if (ostr != NULL) (* ostr) << "could not open file " << buffer << endl;
	if (ostr != NULL) (* ostr) << "found " << lj_vector.size() << " lj-datasets." << endl;
	file.close();
	
	ostrstream cifn(buffer, sizeof(buffer));
	cifn << path << "parameters5.txt" << ends; file.open(buffer, ios::in);
	if (file.good())
	{
		if (ostr != NULL) (* ostr) << "reading file \"" << buffer << "\": ";
		while (file.peek() != 'e')
		{
			if (file.peek() == '0')
			{
				exp1_ci tmp; char bt[16];
				file >> tmp.atmtp[0] >> tmp.atmtp[1] >> bt >> tmp.delta;
				
				tmp.bndtp = bondtype(bt[0]);
				ci_vector.push_back(tmp);
			}
			
			file.getline(buffer, sizeof(buffer));
		}
	} else if (ostr != NULL) (* ostr) << "could not open file " << buffer << ": ";
	if (ostr != NULL) (* ostr) << "found " << ci_vector.size() << " ci-datasets." << endl;
	file.close();
}

exp1_tables::~exp1_tables(void)
{
	for (i32u n1 = 0;n1 < at_vector.size();n1++)
	{
		delete at_vector[n1].typerule;
		delete[] at_vector[n1].description;
	}
}

exp1_tables * exp1_tables::GetInstance(void)
{
	if (instance != NULL) return instance;
	else return (instance = new exp1_tables());
}

f32 exp1_tables::GetChargeInc(mm1_bond * ref, ostream * ostr)
{
	i32s atmtp[2];
	for (i32s n1 = 0;n1 < 2;n1++)
	{
		atmtp[n1] = ref->atmr[n1]->atmtp;
	}
	
	for (i32u n1 = 0;n1 < ci_vector.size();n1++)
	{
		if (ci_vector[n1].bndtp.GetValue() != ref->bt.GetValue()) continue;
		
		for (i32s dir = 0;dir < 2;dir++)
		{
			bool test1 = (ci_vector[n1].atmtp[0] == atmtp[dir]);
			bool test2 = (ci_vector[n1].atmtp[1] == atmtp[!dir]);
			
			if (test1 && test2)
			{
				if (!dir) return +ci_vector[n1].delta;
				else return -ci_vector[n1].delta;
			}
		}
	}
	
	if (ostr != NULL)
	{
		(* ostr) << "there was no record for the following ci: ";
		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
		(* ostr) << ref->bt.GetValue() << endl;
	}
	
	return 0.0;
}

bool exp1_tables::Init(mm1_eng * eng, mm1_exp1_bt1 * ref, i32s bt)
{
	i32s atmtp[2];
	for (i32s n1 = 0;n1 < 2;n1++)
	{
		atmtp[n1] = eng->index[ref->atmi[n1]]->atmtp;
	}
	
	for (i32u n1 = 0;n1 < bs_vector.size();n1++)
	{
		if (bs_vector[n1].bndtp.GetValue() != bt) continue;
		
		bool flag = false;
		for (i32s dir = 0;dir < 2;dir++)
		{
			bool test1 = (bs_vector[n1].atmtp[0] == atmtp[dir]);
			bool test2 = (bs_vector[n1].atmtp[1] == atmtp[!dir]);
			
			if (test1 && test2) flag = true;
			
			bool wc1 = (bs_vector[n1].atmtp[0] == 0xFFFF);
			bool wc2 = (bs_vector[n1].atmtp[1] == 0xFFFF);
			
			if (wc1 && test2) flag = true;
			if (test1 && wc2) flag = true;
			if (wc1 && wc2) flag = true;
			
			if (flag) break;
		}
		
		if (flag)
		{
			ref->opt = 0.1 * bs_vector[n1].param[0];
			ref->fc = 418.68 * bs_vector[n1].param[1];
			
			return true;
		}
	}
	
	if (eng->mdl->ostr != NULL)
	{
		(* eng->mdl->ostr) << "unknown bst: ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
		(* eng->mdl->ostr) << bt << endl;
	}
	
	ref->opt = 0.1 * 1.100;
	ref->fc = 418.68 * 500.0;
	
	return false;
}

bool exp1_tables::Init(mm1_eng * eng, mm1_exp1_bt2 * ref, i32s * bt)
{
	i32s atmtp[3];
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		atmtp[n1] = eng->index[ref->atmi[n1]]->atmtp;
	}
	
	for (i32u n1 = 0;n1 < ab_vector.size();n1++)
	{
		if (ab_vector[n1].atmtp[1] != atmtp[1]) continue;
		
		// bondtype checking not yet implemented....
		// it will be basically similar to the above, but must be moved into the dir-loop!!!
		// it will be basically similar to the above, but must be moved into the dir-loop!!!
		// it will be basically similar to the above, but must be moved into the dir-loop!!!
		
		bool flag = false;
		for (i32s dir = 0;dir < 2;dir++)
		{
			i32s index[2];
			index[0] = (!dir ? 0 : 2);
			index[1] = (!dir ? 2 : 0);
			
			bool test1 = (ab_vector[n1].atmtp[0] == atmtp[index[0]]);
			bool test2 = (ab_vector[n1].atmtp[2] == atmtp[index[1]]);
			
			if (test1 && test2) flag = true;
			
			bool wc1 = (ab_vector[n1].atmtp[0] == 0xFFFF);
			bool wc2 = (ab_vector[n1].atmtp[2] == 0xFFFF);
			
			if (wc1 && test2) flag = true;
			if (test1 && wc2) flag = true;
			if (wc1 && wc2) flag = true;
			
			if (flag) break;
		}
		
		if (flag)
		{
			ref->opt = M_PI * ab_vector[n1].param[0] / 180.0;
			ref->fc = 13744.5 * ab_vector[n1].param[1];
			
			return true;
		}
	}
	
	if (eng->mdl->ostr != NULL)
	{
		(* eng->mdl->ostr) << "unknown abn: " << hex;
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[2] << dec << " ";
		(* eng->mdl->ostr) << bt[0] << " " << bt[1] << endl;
	}
	
	ref->opt = M_PI * 120.0 / 180.0;
	ref->fc = 13744.5 * 0.020;
	
	return false;
}

bool exp1_tables::Init(mm1_eng * eng, mm1_exp1_bt3 * ref, i32s * bt)
{
	i32s atmtp[4];
	for (i32s n1 = 0;n1 < 4;n1++)
	{
		atmtp[n1] = eng->index[ref->atmi[n1]]->atmtp;
	}
	
	for (i32u n1 = 0;n1 < tr_vector.size();n1++)
	{
		if (tr_vector[n1].bndtp[1].GetValue() != bt[1]) continue;

		// proper bondtype checking not yet implemented....
		// it will be basically similar to the above, but must be moved into the dir-loop!!!
		// it will be basically similar to the above, but must be moved into the dir-loop!!!
		// it will be basically similar to the above, but must be moved into the dir-loop!!!
		
		bool flag = false;
		for (i32s dir = 0;dir < 2;dir++)
		{
			i32s index[4];
			index[0] = (!dir ? 0 : 3);
			index[1] = (!dir ? 1 : 2);
			index[2] = (!dir ? 2 : 1);
			index[3] = (!dir ? 3 : 0);
			
			bool test1 = (tr_vector[n1].atmtp[0] == atmtp[index[0]]);
			bool test2 = (tr_vector[n1].atmtp[1] == atmtp[index[1]]);
			bool test3 = (tr_vector[n1].atmtp[2] == atmtp[index[2]]);
			bool test4 = (tr_vector[n1].atmtp[3] == atmtp[index[3]]);
			
			if (test1 && test2 && test3 && test4) flag = true;
			
			bool wc1 = (tr_vector[n1].atmtp[0] == 0xFFFF);
			bool wc2 = (tr_vector[n1].atmtp[3] == 0xFFFF);
			
			if (wc1 && test2 && test3 && test4) flag = true;
			if (test1 && test2 && test3 && wc2) flag = true;
			if (wc1 && test2 && test3 && wc2) flag = true;
			
			if (flag) break;
		}
		
		if (flag)
		{
			ref->k = 4.1868 * tr_vector[n1].k;
			ref->s = tr_vector[n1].s;
			
			return true;
		}
	}
	
	if (eng->mdl->ostr != NULL)
	{
		(* eng->mdl->ostr) << "unknown tor: " << hex;
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[2] << dec << " ";
		(* eng->mdl->ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[3] << dec << " ";
		(* eng->mdl->ostr) << bt[0] << " " << bt[1] << " " << bt[2] << endl;
	}
	
	ref->k = 0.0;
	ref->s = +1.0;
	
	return false;
}

bool exp1_tables::Init(mm1_eng * eng, mm1_exp1_nbt1 * ref, bool is14)
{
	i32s atmtp[2];
	for (i32s n1 = 0;n1 < 2;n1++)
	{
		atmtp[n1] = eng->index[ref->atmi[n1]]->atmtp;
	}
	
	i32u index[2];
	for (i32s n1 = 0;n1 < 2;n1++)
	{
		index[n1] = 0;
		
		while (index[n1] < lj_vector.size())
		{
			if (lj_vector[index[n1]].type == atmtp[n1]) break;
			else index[n1]++;
		}
		
		if (index[n1] == lj_vector.size())
		{
			cout << "bad atomtype!!! using hydrogen instead..." << endl;
			index[n1] = 0;
		}
	}
	
	f32 energy = 4.1868 * sqrt(lj_vector[index[0]].k * lj_vector[index[1]].k);
	f32 optdist = 0.1 * (lj_vector[index[0]].r + lj_vector[index[1]].r);
	
	f32 charge1 = eng->index[ref->atmi[0]]->charge;
	f32 charge2 = eng->index[ref->atmi[1]]->charge;
	ref->qq = 138.9354518 * charge1 * charge2;
	
	if (is14)
	{
		energy *= 0.5;
		ref->qq *= 0.5;
	}
	
	ref->k1 = optdist * pow(1.0 * energy, 1.0 / 12.0);
	ref->k2 = optdist * pow(2.0 * energy, 1.0 / 6.0);
	
	return true;
}

i32s exp1_tables::UpdateTypes(mm1_mdl * mdl)
{
	i32s errors = 0;
	if (mdl->ostr != NULL) (* mdl->ostr) << "setting up atom types..." << endl;
	for (iter_mm1al it1 = mdl->atom_list.begin();it1 != mdl->atom_list.end();it1++)
	{
		if (mdl->ostr != NULL && !((* it1).index % 10)) (* mdl->ostr) << "*" << flush;
		
		i32u at_range[2];
		at_range[0] = 0; while (at_range[0] != at_vector.size() && (at_vector[at_range[0]].atomtype >> 8) != (* it1).el.GetAtomicNumber()) at_range[0]++;
		at_range[1] = at_range[0]; while (at_range[1] != at_vector.size() && (at_vector[at_range[1]].atomtype >> 8) == (* it1).el.GetAtomicNumber()) at_range[1]++;
		
		i32s index = NOT_DEFINED; bool flag;
		for (i32u n1 = at_range[0];n1 < at_range[1];n1++)
		{
			if (!at_vector[n1].typerule->sr_vector.size()) flag = true;	// empty rule???
			else flag = at_vector[n1].typerule->Check(mdl, & (* it1), 0);
			if (flag) index = n1;
		}
		
		if (index != NOT_DEFINED)
		{
			(* it1).atmtp = at_vector[index].atomtype;
		}
		else
		{
			if (mdl->ostr != NULL) (* mdl->ostr) << "problem: unknown atomtype!!!" << endl;
			(* it1).atmtp = NOT_DEFINED; (* it1).selected = true; errors++;
		}
	}
	
	if (mdl->ostr != NULL) (* mdl->ostr) << endl;
	return errors;
}

void exp1_tables::PrintAllTypeRules(ostream & p1)
{
	for (i32u n1 = 0;n1 < at_vector.size();n1++)
	{
		p1 << (n1 + 1) << ": 0x" << hex << setw(4) << setfill('0') << at_vector[n1].atomtype << dec;
		p1 << " (" << (* at_vector[n1].typerule) << ") \"" << at_vector[n1].description << "\"" << endl;
	}
	
	p1 << at_vector.size() << " entries." << endl;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
