// MODEL.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "model.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

bs_mdl_simple::bs_mdl_simple(ostream * p1)
{
	ostr = p1;
}

bs_mdl_simple::~bs_mdl_simple(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

bs_mdl_extended::bs_mdl_extended(ostream * p1) : bs_mdl_simple(p1)
{
//	cs_vector.push_back();			// this should work...
	cs_vector.push_back(crd_set());		// ...but this is safer?!?!?!
	cs_vector[0].visible = true;
}

bs_mdl_extended::~bs_mdl_extended(void)
{
}

void bs_mdl_extended::UpdateAccumValues(void)		// or NormalizeAccumValues???
{
	f32 sum = 0.0;
	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
	{
		if (cs_vector[n1].visible)
		{
			sum += cs_vector[n1].accum_weight;
		}
	}
	
	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
	{
		if (cs_vector[n1].visible)
		{
			fGL tmp1 = cs_vector[n1].accum_weight / sum;
			cs_vector[n1].accum_value = tmp1;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

crd_set::crd_set(void)
{
	SetDescription(NULL);
	
	accum_weight = 1.0;		// accum_value is not updated here?!?!?!?!?!
	visible = false;
}

crd_set::crd_set(const crd_set & p1)
{
	SetDescription(p1.description);
	
	accum_weight = p1.accum_weight;
	accum_value = p1.accum_value;
	visible = p1.visible;
}

crd_set::~crd_set(void)
{
	if (description != NULL) delete[] description;
}

void crd_set::SetDescription(const char * p1)
{
	if (p1 != NULL)
	{
		description = new char[strlen(p1) + 1];
		strcpy(description, p1);
	}
	else description = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

superimpose::superimpose(i32s p1, i32s p2) : conjugate_gradient(Newton1Num, 0.0001, 30)
{
	index[0] = p1;
	index[1] = p2;
	
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		rot[n1] = drot[n1] = 0.0; AddVar(& rot[n1], & drot[n1]);
		loc[n1] = dloc[n1] = 0.0; AddVar(& loc[n1], & dloc[n1]);
	}
}

superimpose::~superimpose(void)
{
}

// this kind of fitting may fail if the angles are not near zero?????!?!?!
// this kind of fitting may fail if the angles are not near zero?????!?!?!
// this kind of fitting may fail if the angles are not near zero?????!?!?!

void superimpose::GetMatrix(fGL * p1) const
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix(); glLoadIdentity();
	
	// here angles are multiplied by 1000.0 to get same "sensitivity"...
	// here angles are multiplied by 1000.0 to get same "sensitivity"...
	// here angles are multiplied by 1000.0 to get same "sensitivity"...
	
	glRotated(1000.0 * rot[0], 1.0, 0.0, 0.0);
	glRotated(1000.0 * rot[1], 0.0, 1.0, 0.0);
	glRotated(1000.0 * rot[2], 0.0, 0.0, 1.0);
	glTranslated(loc[0], loc[1], loc[2]);
	
	glGetFloatv(GL_MODELVIEW_MATRIX, p1);  
	glPopMatrix();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
