// MTOOLS.CPP

// Copyright (C) 1998 Tommi Hassinen, Jarno Huuskonen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef TARGET1
#include "mtools.h"

#include "window.h"
#include "guibase.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

transformer mouse_tool::tool_transformer = transformer();
i32s mouse_tool::latest_x = NOT_DEFINED; i32s mouse_tool::latest_y = NOT_DEFINED;
fGL mouse_tool::ang_sensitivity = 180.0; fGL mouse_tool::dist_sensitivity = 2.0;

mouse_tool::mouse_tool(void)
{
}

mouse_tool::~mouse_tool(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void draw_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	latest_x = x; latest_y = y;
	win->cam->RenderWindow(win, camera::Draw, x, y);
}

void draw_tool::MotionEvent(window *, i32s, i32s)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void select_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state != mouse_tool::Down) return;
	win->cam->RenderWindow(win, camera::Select, x, y);
}

void select_tool::MotionEvent(window *, i32s, i32s)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void orbit_xy_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state == mouse_tool::Down)
	{
		if (win->quick_update)		// if quick, change the rendering params...
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		
		if (!window::shift_down)
		{
			ref_to_object = win->cam; local_object = true;
		}
		else
		{
			ref_to_object = win->gui->selected_object;
			
			i32s index = win->gui->IsLight(ref_to_object);
			local_object = (index != NOT_DEFINED && win->gui->light_vector[index]->owner == win->cam);
			
			if (!ref_to_object && window::shift_down)
			{
				tool_transformer.Init(win->gui);
				ref_to_object = (& tool_transformer);
			}
			
			if (ref_to_object) ref_to_object->BeginTransformation();
		}
		
		rdata = win->cam;
		
		latest_x = x;
		latest_y = y;
	}
	else
	{
		if (ref_to_object) ref_to_object->EndTransformation();
		
		if (win->quick_update)		// if quick, put back original params and update...
		{
			win->render = render;
			win->accumulate = accumulate;
			
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
}

void orbit_xy_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL ang[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && ref_to_object)
	{
		ang[0] = ang_sensitivity * (fGL) (latest_y - y) / (fGL) win->size[1];
		ang[1] = ang_sensitivity * (fGL) (x - latest_x) / (fGL) win->size[0];
		ref_to_object->OrbitObject(ang, * rdata);
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else
		{
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void orbit_z_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state == mouse_tool::Down)
	{
		if (win->quick_update)		// if quick, change the rendering params...
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		
		if (!window::shift_down)
		{
			ref_to_object = win->cam; local_object = true;
		}
		else
		{
			ref_to_object = win->gui->selected_object;
			
			i32s index = win->gui->IsLight(ref_to_object);
			local_object = (index != NOT_DEFINED && win->gui->light_vector[index]->owner == win->cam);
			
			if (!ref_to_object && window::shift_down)
			{
				tool_transformer.Init(win->gui);
				ref_to_object = (& tool_transformer);
			}
			
			if (ref_to_object) ref_to_object->BeginTransformation();
		}
		
		rdata = win->cam;
		
		latest_x = x;
		latest_y = y;
	}
	else
	{
		if (ref_to_object) ref_to_object->EndTransformation();
		
		if (win->quick_update)		// if quick, put back original params and update...
		{
			win->render = render;
			win->accumulate = accumulate;
			
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
}

void orbit_z_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL ang[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && ref_to_object)
	{
		ang[2] = ang_sensitivity * (fGL) (x - latest_x) / (fGL) win->size[0];
		ref_to_object->OrbitObject(ang, * rdata);
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else
		{
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void rotate_xy_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state == mouse_tool::Down)
	{
		if (win->quick_update)		// if quick, change the rendering params...
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		
		if (!window::shift_down)
		{
			ref_to_object = win->cam; local_object = true;
		}
		else
		{
			ref_to_object = win->gui->selected_object;
			
			i32s index = win->gui->IsLight(ref_to_object);
			local_object = (index != NOT_DEFINED && win->gui->light_vector[index]->owner == win->cam);
			
			if (!ref_to_object && window::shift_down)
			{
				tool_transformer.Init(win->gui);
				ref_to_object = (& tool_transformer);
			}
			
			if (ref_to_object) ref_to_object->BeginTransformation();
		}
		
		rdata = win->cam;
		
		latest_x = x;
		latest_y = y;
	}
	else
	{
		if (ref_to_object) ref_to_object->EndTransformation();
		
		if (win->quick_update)		// if quick, put back original params and update...
		{
			win->render = render;
			win->accumulate = accumulate;
			
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
}

void rotate_xy_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL ang[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && ref_to_object)
	{
		ang[0] = ang_sensitivity * (fGL) (latest_y - y) / (fGL) win->size[1];
		ang[1] = ang_sensitivity * (fGL) (x - latest_x) / (fGL) win->size[0];
		ref_to_object->RotateObject(ang, * rdata);
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else
		{
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void rotate_z_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state == mouse_tool::Down)
	{
		if (win->quick_update)		// if quick, change the rendering params...
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		
		if (!window::shift_down)
		{
			ref_to_object = win->cam; local_object = true;
		}
		else
		{
			ref_to_object = win->gui->selected_object;
			
			i32s index = win->gui->IsLight(ref_to_object);
			local_object = (index != NOT_DEFINED && win->gui->light_vector[index]->owner == win->cam);
			
			if (!ref_to_object && window::shift_down)
			{
				tool_transformer.Init(win->gui);
				ref_to_object = (& tool_transformer);
			}
			
			if (ref_to_object) ref_to_object->BeginTransformation();
		}
		
		rdata = win->cam;
		
		latest_x = x;
		latest_y = y;
	}
	else
	{
		if (ref_to_object) ref_to_object->EndTransformation();
		
		if (win->quick_update)		// if quick, put back original params and update...
		{
			win->render = render;
			win->accumulate = accumulate;
			
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
}

void rotate_z_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL ang[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && ref_to_object)
	{
		ang[2] = ang_sensitivity * (fGL) (x - latest_x) / (fGL) win->size[0];
		ref_to_object->RotateObject(ang, * rdata);
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else
		{
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void translate_xy_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state == mouse_tool::Down)
	{
		if (win->quick_update)		// if quick, change the rendering params...
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		
		if (!window::shift_down)
		{
			ref_to_object = win->cam; local_object = true;
		}
		else
		{
			ref_to_object = win->gui->selected_object;
			
			i32s index = win->gui->IsLight(ref_to_object);
			local_object = (index != NOT_DEFINED && win->gui->light_vector[index]->owner == win->cam);
			
			if (!ref_to_object && window::shift_down)
			{
				tool_transformer.Init(win->gui);
				ref_to_object = (& tool_transformer);
			}
			
			if (ref_to_object) ref_to_object->BeginTransformation();
		}
		
		// if CTRL is down, use object's own direction, not that of camera's !!!!!!!!!
		// if CTRL is down, use object's own direction, not that of camera's !!!!!!!!!
		// if CTRL is down, use object's own direction, not that of camera's !!!!!!!!!
		
		if (window::ctrl_down) tdata = ref_to_object->GetLocData();
		else tdata = win->cam->GetLocData();
		
		latest_x = x;
		latest_y = y;
	}
	else
	{
		if (ref_to_object) ref_to_object->EndTransformation();
		
		if (win->quick_update)		// if quick, put back original params and update...
		{
			win->render = render;
			win->accumulate = accumulate;
			
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
}

void translate_xy_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL dist[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && ref_to_object && tdata)
	{
		dist[0] = dist_sensitivity * win->range[0] * (fGL) (latest_x - x) / (fGL) win->size[0];
		dist[1] = dist_sensitivity * win->range[1] * (fGL) (latest_y - y) / (fGL) win->size[1];
		ref_to_object->TranslateObject(dist, tdata);
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else
		{
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void translate_z_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (window::state == mouse_tool::Down)
	{
		if (win->quick_update)		// if quick, change the rendering params...
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		
		if (!window::shift_down)
		{
			ref_to_object = win->cam; local_object = true;
		}
		else
		{
			ref_to_object = win->gui->selected_object;
			
			i32s index = win->gui->IsLight(ref_to_object);
			local_object = (index != NOT_DEFINED && win->gui->light_vector[index]->owner == win->cam);
			
			if (!ref_to_object && window::shift_down)
			{
				tool_transformer.Init(win->gui);
				ref_to_object = (& tool_transformer);
			}
			
			if (ref_to_object) ref_to_object->BeginTransformation();
		}
		
		// if CTRL is down, use object's own direction, not that of camera's !!!!!!!!!
		// if CTRL is down, use object's own direction, not that of camera's !!!!!!!!!
		// if CTRL is down, use object's own direction, not that of camera's !!!!!!!!!
		
		if (window::ctrl_down) tdata = ref_to_object->GetLocData();
		else tdata = win->cam->GetLocData();
		
		latest_x = x;
		latest_y = y;
	}
	else
	{
		if (ref_to_object) ref_to_object->EndTransformation();
		
		if (win->quick_update)		// if quick, put back original params and update...
		{
			win->render = render;
			win->accumulate = accumulate;
			
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
}

void translate_z_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL dist[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && ref_to_object && tdata)
	{
		dist[2] = dist_sensitivity * win->range[1] * (fGL) (latest_y - y) / (fGL) win->size[1];
		ref_to_object->TranslateObject(dist, tdata);
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else
		{
			if (local_object) win->gui->UpdateWindows(win->cam);
			else win->gui->UpdateAllWindows();
		}
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void zoom_tool::ButtonEvent(window * win, i32s x, i32s y)
{
	if (win->quick_update && win && win->gui && win->cam)
	{
		if (window::state == mouse_tool::Down)
		{
			render = win->render;
			accumulate = win->accumulate;
			
			win->render = RENDER_WIREFRAME;
			win->accumulate = false;
		}
		else
		{
			win->render = render;
			win->accumulate = accumulate;
			
			win->gui->UpdateWindows(win->cam);
		}
	}
	
	latest_x = x;
	latest_y = y;
}

void zoom_tool::MotionEvent(window * win, i32s x, i32s y)
{
	fGL dist[3] = { 0.0, 0.0, 0.0 };
	if (win && win->gui && win->cam)
	{
		dist[2] = dist_sensitivity * win->range[1] * (fGL) (latest_y - y) / (fGL) win->size[1];
		win->cam->TranslateObject(dist, win->cam->GetLocData());
		win->cam->focus += dist[2];
		
		if (win->quick_update) win->gui->UpdateWindow(win);
		else win->gui->UpdateWindows(win->cam);
	}
	
	latest_x = x;
	latest_y = y;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// this is under construction...
// the idea is that "control tool" would change properties of some objects, for example
// dimension of a plane/surface, brightness or cutoff/exponent of a spotlight etc...

void control_tool::ButtonEvent(window * win, i32s x, i32s y)
{
}

void control_tool::MotionEvent(window * win, i32s x, i32s y)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET1

// eof
