// QM1GUI.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef TARGET1
#include "qm1gui.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

//#include <fstream.h>
//#include <strstream.h>

#ifdef TARGET3
#include "t3main.h"
#endif	// TARGET3

////////////////////////////////////////////////////////////////////////////////////////////////////

qm1_gui::qm1_gui(window_factory & p1, ostream * p2) : bs_gui(p1), qm1_mdl(p2)
{
}

qm1_gui::~qm1_gui(void)
{
}

const char * qm1_gui::GetType(void)
{
	return "QM, atomic calculations";
}

color_mode * qm1_gui::GetDefaultColorMode(void)
{
	return NULL;
}

void qm1_gui::SelectAll(void)
{
}

void qm1_gui::InvertSelection(void)
{
}

// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!

void qm1_gui::InitGL(void)
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
	
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	const fGL specular_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	const fGL ambient_intensity[4] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
	
	glFogi(GL_FOG_MODE, GL_EXP); glFogf(GL_FOG_DENSITY, 0.15);
	const fGL fog_color[4] = { 0.0, 0.0, 0.0, 0.0 };
	glFogfv(GL_FOG_COLOR, fog_color);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glSelectBuffer(SB_SIZE, select_buffer);
}

void qm1_gui::Render(window * win, rmode rm)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	if (win->enable_fog) glEnable(GL_FOG);
	
	glPointSize(3.0); glLineWidth(1.0);
	
	glBegin(GL_POINTS);
	glColor3fv(el.GetColor());
	glVertex3f(0.0, 0.0, 0.0);
	glEnd();
	
	if (rm != Transform2) win->cam->RenderObjects(win);
	if (win->enable_fog) glDisable(GL_FOG);
}

void qm1_gui::Center(transformer *)
{
}

void qm1_gui::Transform(transformer *)
{
}

void qm1_gui::DrawEvent(window *, vector<iGLu> &)
{
}

void qm1_gui::SelectEvent(window *, vector<iGLu> &)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET2

void qm1_gui::KeyEvent(window * win, char key, int, int)
{
//	fGL tmp1; fGL tmp2;
//	i32s sz[3] = { 50, 50, 50 };
//	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	switch (key)
	{
		case 'h':
		cout << "+ -       -> change the element" << endl;
		break;
		
		case '+': ++el;
		cout << "element = " << el.GetAtomicNumber();
		cout << " " << el.GetSymbol() << endl;
		break;
		
		case '-': --el;
		cout << "element = " << el.GetAtomicNumber();
		cout << " " << el.GetSymbol() << endl;
		break;
		
/*		case 'e': tmp1 = 0.1;
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp2;
		AddObject(new color_plane(ol_static(), 30, tmp2, this, (ValueFunction *) qm1_exact_GetObjValue, GetRGBRange2, tmp1, "ESP-"));
		UpdateAllWindows();
		break;
		
		case 'n': tmp1 = 4.1868 * 33.20716;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? "; cin >> tmp2;
		dm[0] = dm[1] = dm[2] = tmp2;
		
		AddObject(new color_surface(ol_static(), sz, dm, this, (ValueFunction *) mm1_GetESPValue, GetUnity, GetRedColor, +tmp1, 1.0, 1.0e-6 * tmp1, 1000, false, "ESP-", ostr));
		AddObject(new color_surface(ol_static(), sz, dm, this, (ValueFunction *) mm1_GetESPValue, GetUnity, GetBlueColor, -tmp1, 1.0, 1.0e-6 * tmp1, 1000, false, "ESP-", ostr));
		UpdateAllWindows();
		break;*/
		
	}
}

#endif	// TARGET2

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3

void qm1_gui::PopupEvent(window * win, int signal)
{
	switch (signal)
	{
	}
}

#endif	// TARGET3

////////////////////////////////////////////////////////////////////////////////////////////////////
/*
fGL GetESPValue(fGL * crd, void * ref, fGL * gdt)
{
	mm1_gui * gui = (mm1_gui *) ref; mm1_mdl * mdl = (mm1_mdl *) gui;
	
	fGL sum1 = 0.0;
	if (gdt != NULL) gdt[0] = gdt[1] = gdt[2] = 0.0;
	for (iter_mm1al it1 = mdl->atom_list.begin();it1 != mdl->atom_list.end();it1++)
	{
		fGL tmp1[3]; fGL sum2 = 0.0;
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			tmp1[n1] = crd[n1] - (* it1).crd[0][n1];
			sum2 += tmp1[n1] * tmp1[n1];
		}
		
		if (sum2 == 0.0) return +1.0e+35;	// numeric_limits<fGL>::max() !??!?!
		
		// do we have a correct constant here???
		// do we have a correct constant here???
		// do we have a correct constant here???
		
		sum2 = sqrt(sum2);
		fGL tmp2 = 4.1868 * 33.20716 * (* it1).charge / sum2;
		sum1 += tmp2;
		
		if (gdt != NULL)	// sign ??? constant ???
		{
			for (i32s n1 = 0;n1 < 3;n1++)
			{
				fGL tmp3 = tmp1[n1] / sum2;
				fGL tmp4 = tmp3 * tmp2 / sum2;
				gdt[n1] += tmp4;
			}
		}
	}
	
	return sum1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// f = sum[(r/a)^-3] = sum[a^3 * r^-3]
// df/dr = -3 * sum[a^3 * r^-4]

fGL GetSASValue(fGL * crd, void * ref, fGL * gdt)
{
	mm1_gui * gui = (mm1_gui *) ref; mm1_mdl * mdl = (mm1_mdl *) gui;
	
	fGL sum1 = 0.0; if (gdt != NULL) gdt[0] = gdt[1] = gdt[2] = 0.0;
	for (iter_mm1al it1 = mdl->atom_list.begin();it1 != mdl->atom_list.end();it1++)
	{
		fGL tmp1[3]; fGL sum2 = 0.0;
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			tmp1[n1] = crd[n1] - (* it1).crd[0][n1];
			sum2 += tmp1[n1] * tmp1[n1];
		}
		
		if (sum2 == 0.0) return +1.0e+35;	// numeric_limits<fGL>::max() ?!?!!?
		
		// calculate r to sum2 and f to tmp3...
		// calculate r to sum2 and f to tmp3...
		// calculate r to sum2 and f to tmp3...
		
		sum2 = sqrt(sum2);
		fGL tmp2 = sum2 / (* it1).vdwr;
		fGL qqq = tmp2 * tmp2 * tmp2 * tmp2;
		fGL tmp3 = 1.0 / (qqq * qqq * qqq);
		sum1 += tmp3;
		
		// calculate dr/dx to tmp4 and components to tmp5.
		// does tmp5 have sign/constant errors !?!?!?!
		
		if (gdt != NULL)
		{
			for (i32s n1 = 0;n1 < 3;n1++)
			{
				fGL tmp4 = tmp1[n1] / sum2;
				fGL tmp5 = tmp4 * tmp3 / tmp2;
				gdt[n1] += tmp5;
			}
		}
	}
	
	return sum1;
}
*/

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET1

// eof
