// SURFACE.H : a class for drawing colored 3D-surfaces.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SURFACE_H
#define SURFACE_H

#ifndef TARGET1

struct cs_line;
struct cs_triangle;

struct cs_vertex;

struct cs_param;
class color_surface;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "viewing.h"

#include BEGIN_INCLUDE_STL	// this is compiler-dependent...
#include <vector>
#include <algorithm>
#include END_INCLUDE_STL	// this is compiler-dependent...

////////////////////////////////////////////////////////////////////////////////////////////////////

struct cs_line
{
	i32s index[2];
	
	void Arrange(void)
	{
		sort(index, index + 2);
	}
	
	bool operator==(const cs_line & p1) const
	{
		if (index[0] != p1.index[0]) return false;
		if (index[1] != p1.index[1]) return false;
		return true;
	}
	
	bool operator<(const cs_line & p1) const
	{
		if (index[0] != p1.index[0]) return index[0] < p1.index[0];
		if (index[1] != p1.index[1]) return index[1] < p1.index[1];
		return false;
	}
};

struct cs_triangle
{
	i32s index[3];
};

struct cs_vertex
{
	fGL crd[3]; fGL normal[3];
	fGL color[3]; i32s id;
	
	void Render(fGL alpha, bool nf)
	{
		glColor4f(color[0], color[1], color[2], alpha);
		if (nf) glNormal3fv(normal); glVertex3fv(crd);
	}
};

////////////////////////////////////////////////////////////////////////////////////////////////////

struct cs_param
{
	i32s * np; fGL * dim;
	bool transparent;
	
	bs_gui * gui;
	
	void * ref;
	ValueFunction * vf1;
	ValueFunction * vf2;
	ColorFunction * cf;
	
	fGL svalue;
	fGL cvalue;
	fGL toler;
	i32s maxc;
	
	bool wireframe;
	
	ostream * ostr;
	const char * descr;
};

class color_surface : public smart_object
{
	public:
	
	bs_gui * gui; ostream * ostr;
	i32s np[3]; fGL dim[3];
	
	ValueFunction * GetSurfaceValue;
	ValueFunction * GetColorValue;
	ColorFunction * GetColor;
	
	fGL surface_value; fGL color_value;
	fGL tolerance; i32s max_cycles;
	
	fGL * dist[3];
	vector<bool> grid;
	
	i32s range[3][2];
	vector<cs_vertex> vdata;
	vector<cs_triangle> tdata;
	vector<cs_line> ldata;
	
	bool wireframe;
	v3d<fGL> xdir;
	
	private:
	
	char name[256];
	bool transform_in_progress;
	
	public:
	
	color_surface(const object_location &, cs_param &);
	~color_surface(void);
	
	void SetDimension(fGL *);
	void GetCRD(i32s *, fGL *);
	
	const char * GetObjectName(void) { return name; }	// virtual
	
	bool BeginTransformation(void);			// virtual
	bool EndTransformation(void);			// virtual
	
	void OrbitObject(const fGL *, const camera &);		// virtual
	void RotateObject(const fGL *, const camera &);		// virtual
	
	void TranslateObject(const fGL *, const obj_loc_data *);	// virtual
	
	void SetIndex(i32s, i32s *);
	void SetEdges(i32s, i32s *, i32s *);
	void Interpolate(i32s, i32s *, i32s *);
	
	void Update(void);
	void Render(void);	// virtual...
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET1
#endif	// SURFACE_H

// eof
