// T3DIALOG.CPP

// Copyright (C) 1999 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3
#include "t3dialog.h"

#include <ctype.h>		// ???
#include <fstream.h>		// ???

#include "mm1gui.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

void FileSelection(char * caption, char * filename, GtkSignalFunc sf)
{
	GtkWidget * widget = gtk_file_selection_new(caption);
	if (filename != NULL) gtk_file_selection_set_filename(GTK_FILE_SELECTION(widget), filename);
	
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(widget)->ok_button), "clicked", sf, widget);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(widget)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(widget));
	
	gtk_widget_show(widget);
}

void OpenMM1OkHandler(GtkWidget *, GtkFileSelection * fs)
{
	char * filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	
	mm1_gui * gui = new mm1_gui(* gtkgl_factory::GetInstance(), & cout);
	ifstream ifile(filename, ios::in); gui->ReadStream(ifile); ifile.close();
	gui->AddCamera(4.0);
	
	gtk_widget_destroy(GTK_WIDGET(fs));
}

void OpenMM2OkHandler(GtkWidget *, GtkFileSelection * fs)
{
	char * filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	gtk_widget_destroy(GTK_WIDGET(fs));
}

void ImportMM1OkHandler(GtkWidget *, GtkFileSelection * fs)
{
	char * filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	
	mm1_gui * gui = new mm1_gui(* gtkgl_factory::GetInstance(), & cout);
	
	switch (tolower(filename[strlen(filename) - 1]))
	{
		case 'n':	// *.hin
		gui->ImportHIN(filename);
		break;
		
		case 't':	// *.ent
		case 'b':	// *.pdb
		gui->ImportENT(filename, NOT_DEFINED);
		break;
		
		default:
		cout << "unkwown extension. please try one of the following: *.HIN *.ENT *.PDB" << endl;
	}
	
	gui->AddCamera(4.0);
	gtk_widget_destroy(GTK_WIDGET(fs));
}

void ImportMM2OkHandler(GtkWidget *, GtkFileSelection * fs)
{
	char * filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	gtk_widget_destroy(GTK_WIDGET(fs));
}

////////////////////////////////////////////////////////////////////////////////////////////////////

about_dialog::about_dialog(void)
{
	GtkWidget * window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	gtk_window_set_title(GTK_WINDOW(window), "About The Program");
	gtk_container_set_border_width(GTK_CONTAINER(window), 2);
	
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", (GtkSignalFunc) gtk_main_quit, NULL);
	
	GtkWidget * vbox = gtk_vbox_new(FALSE, 2);
	
	char txtbuffer[1024];
	ostrstream str(txtbuffer, sizeof(txtbuffer));
	
	str << "Ghemical for GTK+" << endl;
	str << "Version " << VERSION << " released on " << RELEASEDATE << endl;
	str << endl;
	str << "For more information please visit " << WEBSITE << endl;
	str << endl;
	str << "Copyright (C) 1998 Tommi Hassinen." << endl;
	str << endl;
	str << "This program is free software; you can redistribute it and/or" << endl;
	str << "modify it under the terms of the GNU General Public License" << endl;
	str << "as published by the Free Software Foundation; either version 2" << endl;
	str << "of the License, or any later version." << endl;
	str << endl;
	str << "This program is distributed in the hope that it will be useful," << endl;
	str << "but WITHOUT ANY WARRANTY; without even the implied warranty of" << endl;
	str << "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the" << endl;
	str << "GNU General Public License for more details." << endl;
	str << ends;
	
	GtkWidget * label = gtk_label_new(txtbuffer); gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 2);
	
	GtkWidget * button = gtk_button_new_with_label("Ok"); gtk_widget_show(button);
	
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(window));
	
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, TRUE, 2);
	
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	gtk_widget_show(vbox);
	gtk_widget_show(window);
}

about_dialog::~about_dialog(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET3

// eof
