// T3MAIN.CPP

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3
#include "t3main.h"

#include "t3dialog.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * MakeSubMenu(void *** data, GtkSignalFunc sf)
{
	GtkWidget * menu = gtk_menu_new();
	void ** ref = * data;
	
	do
	{
		char * itemname = (char *) (* (ref++));
		GtkWidget * item;
		
		switch (* itemname)
		{
			case 's':		// submenu
			item = gtk_menu_item_new_with_label(itemname + 1);
			
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), MakeSubMenu(& ref, sf));
			gtk_widget_show(item);
			break;
			
			case 'f':		// function
			item = gtk_menu_item_new_with_label(itemname + 1);
			
//			item = gtk_check_menu_item_new_with_label(itemname + 1);
//			gtk_menu_item_configure(GTK_MENU_ITEM(item), TRUE, FALSE);
			
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_signal_connect(GTK_OBJECT(item), "activate", (GtkSignalFunc) sf, (void *) ref[0]);
			gtk_widget_show(item); ref += 1;
			break;
			
			default:		// separator
			item = gtk_menu_item_new();
			gtk_menu_append(GTK_MENU(menu), item);
			gtk_widget_show(item);
		}
	} while(* ref != NULL);
	
	ref++;
	(* data) = ref;
	
	return menu;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * element_dialog::window;

element_dialog::element_dialog(void)
{
	window = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
	
	GtkWidget * table = gtk_table_new(21, 38, TRUE);
	gtk_container_add(GTK_CONTAINER(window), table);
	
	int attachdata[ELEMENT_SYMBOLS][2] =
	{
		{ 1, 1 }, { 35, 1 }, { 1, 3 }, { 3, 3 }, { 25, 3 },		// B
		{ 27, 3 }, { 29, 3 }, { 31, 3 }, { 33, 3 }, { 35, 3 },		// Ne
		{ 1, 5 }, { 3, 5 }, { 25, 5 }, { 27, 5 }, { 29, 5 },		// P
		{ 31, 5 }, { 33, 5 }, { 35, 5 }, { 1, 7 }, { 3, 7 },		// Ca
		{ 5, 7 }, { 7, 7 }, { 9, 7 }, { 11, 7 }, { 13, 7 },		// Mn
		{ 15, 7 }, { 17, 7 }, { 19, 7 }, { 21, 7 }, { 23, 7 },		// Zn
		{ 25, 7 }, { 27, 7 }, { 29, 7 }, { 31, 7 }, { 33, 7 },		// Br
		{ 35, 7 }, { 1, 9 }, { 3, 9 }, { 5, 9 }, { 7, 9 },		// Zr
		{ 9, 9 }, { 11, 9 }, { 13, 9 }, { 15, 9 }, { 17, 9 },		// Rh
		{ 19, 9 }, { 21, 9 }, { 23, 9 }, { 25, 9 }, { 27, 9 },		// Sn
		{ 29, 9 }, { 31, 9 }, { 33, 9 }, { 35, 9 }, { 1, 11 },		// Cs
		{ 3, 11 }, { 5, 11 }, { 9, 16 }, { 11, 16 }, { 13, 16 },	// Nd
		{ 15, 16 }, { 17, 16 }, { 19, 16 }, { 21, 16 }, { 23, 16 },	// Tb
		{ 25, 16 }, { 27, 16 }, { 29, 16 }, { 31, 16 }, { 33, 16 },	// Yb
		{ 35, 16 }, { 7, 11 }, { 9, 11 }, { 11, 11 }, { 13, 11 },	// Re
		{ 15, 11 }, { 17, 11 }, { 19, 11 }, { 21, 11 }, { 23, 11 },	// Hg
		{ 25, 11 }, { 27, 11 }, { 29, 11 }, { 31, 11 }, { 33, 11 },	// At
		{ 35, 11 }, { 1, 13 }, { 3, 13 }, { 5, 13 }, { 9, 18 },		// Th
		{ 11, 18 }, { 13, 18 }, { 15, 18 }, { 17, 18 }, { 19, 18 },	// Am
		{ 21, 18 }, { 23, 18 }, { 25, 18 }, { 27, 18 }, { 29, 18 },	// Fm
		{ 31, 18 }, { 33, 18 }, { 35, 18 }, { 7, 13 }, { 9, 13 }	// Ha
	};
	
	GtkSignalFunc sf = (GtkSignalFunc) element_dialog::SignalHandler; GtkWidget * button;
	
	for (int n1 = 0;n1 < ELEMENT_SYMBOLS;n1++)
	{
		element tmp_element(n1 + 1);
		button = gtk_button_new_with_label(tmp_element.GetSymbol());
		gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) (n1 + 1));
		gtk_widget_show(button);
		
		int x = attachdata[n1][0]; int y = attachdata[n1][1];
		gtk_table_attach_defaults(GTK_TABLE(table), button, x, x + 2, y, y + 2);
	}
	
	gtk_widget_show(table);
	gtk_widget_show(window);
}

element_dialog::~element_dialog(void)
{
}

void element_dialog::SignalHandler(GtkWidget *, int signal)
{
	element::current_element = element(signal);
	toolbox::GetInstance()->UpdateCurrElementLabel();
	gtk_widget_destroy(window);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * bondtype_dialog::window;

bondtype_dialog::bondtype_dialog(void)
{
	window = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
	
	GtkWidget * table = gtk_table_new((2 + 4), 5, TRUE);
	gtk_container_add(GTK_CONTAINER(window), table);
	
	char bt_symbols[4] = { 'S', 'C', 'D', 'T' };
	
	GtkSignalFunc sf = (GtkSignalFunc) bondtype_dialog::SignalHandler; GtkWidget * button;
	
	for (int n1 = 0;n1 < 4;n1++)
	{
		bondtype tmp_bondtype(bt_symbols[n1]);
		button = gtk_button_new_with_label(tmp_bondtype.GetString());
		gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) bt_symbols[n1]);
		gtk_widget_show(button);
		
		int x = 2; int y = n1 + 1;
		gtk_table_attach_defaults(GTK_TABLE(table), button, x, x + 1, y, y + 1);
	}
	
	gtk_widget_show(table);
	gtk_widget_show(window);
}

bondtype_dialog::~bondtype_dialog(void)
{
}

void bondtype_dialog::SignalHandler(GtkWidget *, int signal)
{
	bondtype::current_bondtype = bondtype((char) signal);
	toolbox::GetInstance()->UpdateCurrBondtypeLabel();
	gtk_widget_destroy(window);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

toolbox * toolbox::instance = NULL;
singleton_cleaner<toolbox> toolbox_cleaner;

toolbox::toolbox(void)
{
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "Ghemical for GTK+");
	gtk_container_set_border_width(GTK_CONTAINER(window), 2);
	
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", (GtkSignalFunc) gtk_main_quit, NULL);
	
	vbox = gtk_vbox_new(FALSE, 2);
	
	// create the menu...
	
	void * mainmenu_data[] =
	{
		(void *) "File",
		
		(void *) "sNew...",
		(void *) "fall atoms", (void *) SIGNAL_FILE_NEW_MM1,
		NULL,		// end of "File/New..."
		
		(void *) "sOpen...",
		(void *) "fall atoms", (void *) SIGNAL_FILE_OPEN_MM1,
		(void *) "freduced", (void *) SIGNAL_FILE_OPEN_MM2,
		NULL,		// end of "File/Open..."
		
		(void *) "sImport...",
		(void *) "fall atoms", (void *) SIGNAL_FILE_IMPORT_MM1,
		(void *) "freduced", (void *) SIGNAL_FILE_IMPORT_MM2,
		NULL,		// end of "File/Open..."
		
		(void *) "",	// separator
		
		(void *) "fQuit", (void*) SIGNAL_FILE_QUIT,
		NULL,		// end of "File"
		
		(void *) "Help",
		(void *) "fAbout...", (void *) SIGNAL_HELP_ABOUT,
		NULL,		// end of "Help"
		NULL		// END!!!!!
	};
	
	mainmenu = gtk_menu_bar_new();
	void ** c1 = mainmenu_data;
	
	do
	{
		char * submenulabel = (char *) (* (c1++));
		GtkWidget * label = gtk_menu_item_new_with_label(submenulabel);
		gtk_menu_bar_append(GTK_MENU_BAR(mainmenu), label); GtkWidget * submenu;
		submenu = MakeSubMenu(& c1, (GtkSignalFunc) toolbox::SignalHandler);
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(label), submenu); gtk_widget_show(label);
	} while((* c1) != NULL);
	
	gtk_widget_show(mainmenu); gtk_box_pack_start(GTK_BOX(vbox), mainmenu, FALSE, FALSE, 1);
	
	// create the buttons... IS THIS AUTOMATIC SYSTEM REASONABLE ?!?!?!?!
	
	void * buttons[] =
	{
		(void *) "Draw", (void*) SIGNAL_TOOL_DRAW,
		(void *) "Select", (void*) SIGNAL_TOOL_SELECT,
		NULL,
		(void *) "Trans XY", (void*) SIGNAL_TOOL_TRANS_XY,
		(void *) "Trans Z", (void*) SIGNAL_TOOL_TRANS_Z,
		NULL,
		(void *) "Orbit XY", (void*) SIGNAL_TOOL_ORBIT_XY,
		(void *) "Orbit Z", (void*) SIGNAL_TOOL_ORBIT_Z,
		NULL,
		(void *) "Rotate XY", (void*) SIGNAL_TOOL_ROTATE_XY,
		(void *) "Rotate Z", (void*) SIGNAL_TOOL_ROTATE_Z,
		NULL,
		(void *) "Zoom", (void*) SIGNAL_TOOL_ZOOM,
		(void *) "Control", (void*) SIGNAL_TOOL_CONTROL,
		NULL, NULL
	};
	
	GtkWidget * hbox; GtkWidget * button;
	GtkSignalFunc sf = (GtkSignalFunc) toolbox::SignalHandler;
	
	vector<GtkWidget *> button_widgets;
	
	void ** c2 = buttons;
	while((* c2) != NULL)
	{
		hbox = gtk_hbutton_box_new();
		
		while((* c2) != NULL)
		{
			button = gtk_toggle_button_new_with_label((gchar *) c2[0]);
			gtk_widget_show(button); gtk_container_add(GTK_CONTAINER(hbox), button);
			
			toggle_button_data bdata;
			bdata.button = button; bdata.signal = (int) c2[1];
			tool_buttons.push_back(bdata);
			
			gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) c2[1]); c2 += 2;
		}
		
		gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 0); gtk_widget_show(hbox);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0); c2++;
	}
	
	current_tool_button_index = 4;
	GtkWidget * ctbw = tool_buttons[current_tool_button_index].button;
	int ctbs = tool_buttons[current_tool_button_index].signal;
	
	gtk_signal_handler_block_by_func(GTK_OBJECT(ctbw), sf, (void *) ctbs);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctbw), TRUE);
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctbw), sf, (void *) ctbs);
	
	GtkWidget * separator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, TRUE, 2);
	gtk_widget_show(separator);
	
	//////////
	
	hbox = gtk_hbutton_box_new();
	
	button = gtk_button_new();
	element_label = gtk_label_new(""); UpdateCurrElementLabel();
	gtk_container_add(GTK_CONTAINER(button), element_label); gtk_widget_show(button);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) SIGNAL_TOOL_ELEMENT);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	
	button = gtk_button_new();
	bondtype_label = gtk_label_new(""); UpdateCurrBondtypeLabel();
	gtk_container_add(GTK_CONTAINER(button), bondtype_label); gtk_widget_show(button);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) SIGNAL_TOOL_BONDTYPE);
	gtk_container_add(GTK_CONTAINER(hbox), button);
	
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 0); gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	gtk_widget_show(vbox);
	gtk_widget_show(window);
}

toolbox::~toolbox(void)
{
	// window???
	// box???
}

toolbox * toolbox::GetInstance(void)
{
	if (instance != NULL) return instance;
	else return (instance = new toolbox());
}

void toolbox::UpdateToolButtons(int signal)
{
	int n1 = 0;
	while (true) { if (tool_buttons[n1].signal != signal) n1++; else break; }
//	GtkWidget * widget = tool_buttons[n1].button;	// ?!?!?!?!
	
	// the only purpose of this blocking/unblocking operation is to tell to the
	// toggle button "now you should change your state" WITHOUT emitting any signals...
	
	GtkSignalFunc sf = (GtkSignalFunc) toolbox::SignalHandler;
	GtkWidget * ctbw = tool_buttons[current_tool_button_index].button;
	int ctbs = tool_buttons[current_tool_button_index].signal;
	
	gtk_signal_handler_block_by_func(GTK_OBJECT(ctbw), sf, (void *) ctbs);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctbw), FALSE);
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctbw), sf, (void *) ctbs);
	gtk_widget_show(ctbw); current_tool_button_index = n1;
}

void toolbox::UpdateCurrElementLabel(void)
{
	gtk_label_set_text(GTK_LABEL(element_label), element::current_element.GetString()); 
	gtk_widget_show(element_label); 
}

void toolbox::UpdateCurrBondtypeLabel(void)
{
	gtk_label_set_text(GTK_LABEL(bondtype_label), bondtype::current_bondtype.GetString()); 
	gtk_widget_show(bondtype_label); 
}

void toolbox::SignalHandler(GtkWidget *, int signal)
{
	bs_gui * gui;
	switch (signal)
	{
		case SIGNAL_TOOL_DRAW:
		window::current_tool = & window::tool_draw;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_SELECT:
		window::current_tool = & window::tool_select;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_TRANS_XY:
		window::current_tool = & window::tool_translate_xy;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_TRANS_Z:
		window::current_tool = & window::tool_translate_z;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_ORBIT_XY:
		window::current_tool = & window::tool_orbit_xy;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_ORBIT_Z:
		window::current_tool = & window::tool_orbit_z;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_ROTATE_XY:
		window::current_tool = & window::tool_rotate_xy;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_ROTATE_Z:
		window::current_tool = & window::tool_rotate_z;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_ZOOM:
		window::current_tool = & window::tool_zoom;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_CONTROL:
		window::current_tool = & window::tool_control;
		cout << "current tool = " << window::current_tool->GetToolName() << endl;
		GetInstance()->UpdateToolButtons(signal);
		break;
		
		case SIGNAL_TOOL_ELEMENT:
		new element_dialog();		// memory leak???
		break;
		
		case SIGNAL_TOOL_BONDTYPE:
		new bondtype_dialog();		// memory leak???
		break;
		
		case SIGNAL_FILE_NEW_MM1:
		gui = new mm1_gui(* gtkgl_factory::GetInstance(), & cout);
		gui->AddCamera(4.0);
		break;
		
		case SIGNAL_FILE_QUIT:
		gtk_main_quit();
		break;
		
		case SIGNAL_FILE_OPEN_MM1:
		FileSelection("Open MM1", "*.mm1", (GtkSignalFunc) OpenMM1OkHandler);
		break;
		
		case SIGNAL_FILE_OPEN_MM2:
		FileSelection("Open MM2", "*.mm2", (GtkSignalFunc) OpenMM2OkHandler);
		break;
		
		case SIGNAL_FILE_IMPORT_MM1:
		FileSelection("Import MM1", "*.hin;*.ent", (GtkSignalFunc) ImportMM1OkHandler);
		break;
		
		case SIGNAL_FILE_IMPORT_MM2:
		FileSelection("Import MM2", "*.ent", (GtkSignalFunc) ImportMM2OkHandler);
		break;
		
		case SIGNAL_HELP_ABOUT:
		new about_dialog();		// memory leak???
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char ** argv)
{
	for (i32s n1 = 1;n1 < argc;n1++)
	{
		if (!strcmp(argv[n1], "-h") || !strcmp(argv[n1], "--help"))
		{
			cout << "command-line help ???" << endl;
			return 0;
		}
		
		// -v --verbose -> allow output to cout?!?!?!?!
		// -v --verbose -> allow output to cout?!?!?!?!
		// -v --verbose -> allow output to cout?!?!?!?!
	}
	
	if (!gdk_gl_query())
	{
		cout << "No OpenGL/Mesa!!!" << endl;
		return 1;
	}
	
	gtk_set_locale();
	gtk_init(& argc, & argv);
	
	toolbox_cleaner.SetInstance(toolbox::GetInstance());
	
	gtk_main();
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET3

// eof
