// T3MAIN.H : main for TARGET3 (the GTK+ version).

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef T3MAIN_H
#define T3MAIN_H

#ifdef TARGET3

class element_dialog;
class bondtype_dialog;

class toolbox;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <strstream.h>

#include "t3window.h"

#include "mm1gui.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

#define SIGNAL_TOOL_DRAW		0x1000
#define SIGNAL_TOOL_SELECT		0x1001
#define SIGNAL_TOOL_TRANS_XY		0x1002
#define SIGNAL_TOOL_TRANS_Z		0x1003
#define SIGNAL_TOOL_ORBIT_XY		0x1004
#define SIGNAL_TOOL_ORBIT_Z		0x1005
#define SIGNAL_TOOL_ROTATE_XY		0x1006
#define SIGNAL_TOOL_ROTATE_Z		0x1007
#define SIGNAL_TOOL_ZOOM		0x1008
#define SIGNAL_TOOL_CONTROL		0x1009

#define SIGNAL_TOOL_ELEMENT		0x1100
#define SIGNAL_TOOL_BONDTYPE		0x1101

#define SIGNAL_FILE_NEW_MM1		0x2000
#define SIGNAL_FILE_NEW_MM2		0x2001
#define SIGNAL_FILE_OPEN_MM1		0x2010
#define SIGNAL_FILE_OPEN_MM2		0x2011
#define SIGNAL_FILE_IMPORT_MM1		0x2020
#define SIGNAL_FILE_IMPORT_MM2		0x2021

#define SIGNAL_FILE_QUIT		0x2100

#define SIGNAL_HELP_ABOUT		0x2500

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

#define SIGNAL_POPUP_VIEWS_CREATE_CAMERA	0x5000
#define SIGNAL_POPUP_VIEWS_CREATE_VIEW		0x5001
#define SIGNAL_POPUP_VIEWS_DELETE_VIEW		0x5002

#define SIGNAL_POPUP_LIGHTS_CREATE		0x5100
#define SIGNAL_POPUP_LIGHTS_SWITCH_LOCAL	0x5101
#define SIGNAL_POPUP_LIGHTS_SWITCH_GLOBAL	0x5102

#define SIGNAL_POPUP_FILE_SAVEAS		0x5200
#define SIGNAL_POPUP_FILE_CLOSE			0x5201

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

#define SIGNAL_POPUP_MM1_HYDROGENS			0x7000
#define SIGNAL_POPUP_MM1_BUILDER			0x7001

#define SIGNAL_POPUP_MM1_RENDER_CMODE_STATE		0x7100
#define SIGNAL_POPUP_MM1_RENDER_CMODE_ELEMENT		0x7101

#define SIGNAL_POPUP_MM1_RENDER_RMODE_WIREFRAME		0x7200
#define SIGNAL_POPUP_MM1_RENDER_RMODE_BALLSTICK		0x7201
#define SIGNAL_POPUP_MM1_RENDER_RMODE_VANDERWAALS	0x7202
#define SIGNAL_POPUP_MM1_RENDER_RMODE_CYLINDERS		0x7203
#define SIGNAL_POPUP_MM1_RENDER_RMODE_NOTHING		0x7204

#define SIGNAL_POPUP_MM1_RENDER_LABEL_ELEMENT		0x7300
#define SIGNAL_POPUP_MM1_RENDER_LABEL_BONDTYPE		0x7301
#define SIGNAL_POPUP_MM1_RENDER_LABEL_NOTHING		0x7302

#define SIGNAL_POPUP_MM1_COMP_GEOMOPT			0x7400
#define SIGNAL_POPUP_MM1_COMP_MOLDYN			0x7401

#define SIGNAL_POPUP_MM1_OBJECTS_RIBBON			0x7500
#define SIGNAL_POPUP_MM1_OBJECTS_ESPPLANE		0x7501
#define SIGNAL_POPUP_MM1_OBJECTS_ESPISOS		0x7502
#define SIGNAL_POPUP_MM1_OBJECTS_VDWS			0x7503

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

#define SIGNAL_POPUP_MM2_RENDER_CMODE_STATE		0x8100
#define SIGNAL_POPUP_MM2_RENDER_CMODE_RESIDUE		0x8101
#define SIGNAL_POPUP_MM2_RENDER_CMODE_SASA		0x8102

#define SIGNAL_POPUP_MM2_RENDER_RMODE_WIREFRAME		0x8200
#define SIGNAL_POPUP_MM2_RENDER_RMODE_BALLSTICK		0x8201
#define SIGNAL_POPUP_MM2_RENDER_RMODE_VANDERWAALS	0x8202
#define SIGNAL_POPUP_MM2_RENDER_RMODE_ACCESSIBLE	0x8203
#define SIGNAL_POPUP_MM2_RENDER_RMODE_CYLINDERS		0x8204
#define SIGNAL_POPUP_MM2_RENDER_RMODE_NOTHING		0x8205

#define SIGNAL_POPUP_MM2_COMP_GEOMOPT			0x8400
#define SIGNAL_POPUP_MM2_COMP_MOLDYN			0x8401

#define SIGNAL_POPUP_MM2_OBJECTS_RIBBON			0x8500
#define SIGNAL_POPUP_MM2_OBJECTS_ESPPLANE		0x8501
#define SIGNAL_POPUP_MM2_OBJECTS_ESPISOS		0x8502
#define SIGNAL_POPUP_MM2_OBJECTS_VDWS			0x8503

#define SIGNAL_DUMMY	0xFFFF

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * MakeSubMenu(void ***, GtkSignalFunc);

////////////////////////////////////////////////////////////////////////////////////////////////////

struct toggle_button_data
{
	GtkWidget * button;
	int signal;
};

class element_dialog
{
	private:
	
	static GtkWidget * window;
	
	public:
	
	element_dialog(void);
	~element_dialog(void);
	
	static void SignalHandler(GtkWidget *, int);
};

class bondtype_dialog
{
	private:
	
	static GtkWidget * window;
	
	public:
	
	bondtype_dialog(void);
	~bondtype_dialog(void);
	
	static void SignalHandler(GtkWidget *, int);
};

class toolbox
{
	protected:
	
	static gtkgl_factory factory;
	
	GtkWidget * window;
	GtkWidget * vbox;
	
	GtkWidget * mainmenu;		// ???
	
	int current_tool_button_index;
	vector<toggle_button_data> tool_buttons;
	
	GtkWidget * element_label;
	GtkWidget * bondtype_label;
	
	private:
	
	static toolbox * instance;
	toolbox(void);
	
	public:
	
	~toolbox(void);
	static toolbox * GetInstance(void);
	
	void UpdateToolButtons(int);  
	void UpdateCurrElementLabel(void);
	void UpdateCurrBondtypeLabel(void);
	
	static void SignalHandler(GtkWidget *, int);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int, char **);

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET3
#endif	// T3MAIN_H

// eof
