// T3WINDOW.CPP

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3
#include "t3window.h"

#include BEGIN_INCLUDE_STL	// this is compiler-dependent...
#include <algorithm>
#include END_INCLUDE_STL	// this is compiler-dependent...

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<gtkgl_window *> gtkgl_window::gtkgl_vector;
GtkWidget * gtkgl_window::latest_popup_owner;

////////////////////////////////////////////////////////////////////////////////////////////////////

gtkgl_window::gtkgl_window(bs_gui * p1, camera * p2) : window(p1, p2)
{
	gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	
	int attrlst[] = { GDK_GL_RGBA, GDK_GL_DEPTH_SIZE, 1, GDK_GL_DOUBLEBUFFER, GDK_GL_NONE };
	glarea = gtk_gl_area_new(attrlst); gtkgl_vector.push_back(this);
	
	gtk_widget_set_usize(glarea, 400, 400);
	
	int events = GDK_EXPOSURE_MASK;
	events |= GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK;
	events |= GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK;
	
	gtk_widget_set_events(glarea, events);
	
	gtk_signal_connect(GTK_OBJECT(glarea), "expose_event", GTK_SIGNAL_FUNC(ExposeHandler), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "motion_notify_event", GTK_SIGNAL_FUNC(MotionNotifyHandler), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "button_press_event", GTK_SIGNAL_FUNC(ButtonHandler), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "button_release_event", GTK_SIGNAL_FUNC(ButtonHandler), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "destroy", GTK_SIGNAL_FUNC(DestroyHandler), NULL);
	gtk_signal_connect(GTK_OBJECT(glarea), "configure_event", GTK_SIGNAL_FUNC(ConfigureHandler), NULL);
	
	gtk_container_add(GTK_CONTAINER(gtkwin), glarea);
	gtk_container_set_border_width(GTK_CONTAINER(gtkwin), 1);
	gtk_widget_show(glarea); gtk_widget_show(gtkwin);
	
	// problem: how to inform the window that it should resize??? using small usize
	// above would be resonable, with resize to practical dimensions here.
	
	SetCurrent(); gui->InitGL();
}

gtkgl_window::~gtkgl_window()
{
	vector<gtkgl_window *>::iterator it1;
	it1 = find(gtkgl_vector.begin(), gtkgl_vector.end(), this);
	gtkgl_vector.erase(it1); gtk_widget_destroy(gtkwin);
}

void gtkgl_window::SetCurrent(void)
{
	bool failed = !gtk_gl_area_make_current(GTK_GL_AREA(glarea));
	if (failed) cout << "SetCurrent(): can't make_current!!!" << endl;
}

void gtkgl_window::SetTitle(const char * p1)
{
	gtk_window_set_title(GTK_WINDOW(gtkwin), p1);
}

void gtkgl_window::Update(bool directly)
{
	if (directly) ExposeEvent();
	else ExposeEvent();	// should emit the update request ???
}

void gtkgl_window::ExposeEvent(void)
{
	SetCurrent(); Render(); glFlush();
	gtk_gl_area_swapbuffers(GTK_GL_AREA(glarea));
}

void gtkgl_window::SignalEvent(int)
{
}

gtkgl_window * gtkgl_window::GetWindow(GtkWidget * widget)
{
	vector<gtkgl_window *>::iterator it1 = gtkgl_vector.begin();
	while (it1 != gtkgl_vector.end())
	{
		vector<gtkgl_window *>::iterator it2 = it1++;
		if ((* it2)->glarea == widget) return (* it2);
	}
	
	return NULL;
}

gint gtkgl_window::ButtonHandler(GtkWidget * widget, GdkEventButton * eb)
{
	gtkgl_window * win = GetWindow(widget);
	if (!win) cout << "Unknown ID in MouseHandler !!!" << endl;
	else
	{
		mouse_tool::button tmpb; i32s tmps1;
		switch (eb->button)
		{
			case 1: tmpb = mouse_tool::Left; tmps1 = GDK_BUTTON1_MASK; break;
			case 2: tmpb = mouse_tool::Middle; tmps1 = GDK_BUTTON2_MASK; break;
			default: tmpb = mouse_tool::Right; tmps1 = GDK_BUTTON3_MASK;
		}
		
		mouse_tool::state tmps2 = (eb->state & tmps1) ? mouse_tool::Up : mouse_tool::Down;
		
		if (tmps2 == mouse_tool::Down)
		{
			if (button == mouse_tool::None)
			{
				if (tmpb == mouse_tool::Right)
				{
					gtk_menu_popup(GTK_MENU(GetWindow(widget)->gui->popupmenu), NULL, NULL, NULL, NULL, eb->button, eb->time);
					latest_popup_owner = widget; return TRUE;
				}
				
				button = tmpb;
				
				shift_down = (eb->state & GDK_SHIFT_MASK) ? true : false;
				ctrl_down = (eb->state & GDK_CONTROL_MASK) ? true : false;
				
				state = mouse_tool::Down;
				current_tool->ButtonEvent(win, eb->x, eb->y);
			}
		}
		else
		{
			if (button == mouse_tool::Left && tmpb != mouse_tool::Left) return TRUE;
			if (button == mouse_tool::Middle && tmpb != mouse_tool::Middle) return TRUE;
			if (button == mouse_tool::Right && tmpb != mouse_tool::Right) return TRUE;
			
			state = mouse_tool::Up;
			current_tool->ButtonEvent(win, eb->x, eb->y);
			button = mouse_tool::None;
		}
	}
	
	return TRUE;
}

gint gtkgl_window::ExposeHandler(GtkWidget * widget, GdkEventExpose *)
{
	GetWindow(widget)->ExposeEvent();
	return FALSE;
}

void gtkgl_window::DestroyHandler(GtkObject * object)
{
	GtkWidget * widget = GTK_WIDGET(object);
	GetWindow(widget)->gui->RemoveWindow(GetWindow(widget));
}

gint gtkgl_window::ConfigureHandler(GtkWidget * widget, GdkEventConfigure *)
{
	GetWindow(widget)->SetCurrent();
	GetWindow(widget)->SetSize(widget->allocation.width, widget->allocation.height);
	return TRUE;
}

gint gtkgl_window::MotionNotifyHandler(GtkWidget * widget, GdkEventMotion * event)
{
	int x; int y; GdkModifierType state;
	if (event->is_hint) gdk_window_get_pointer(event->window, & x, & y, & state);
	else { x = (int) event->x; y = (int) event->y; state = (GdkModifierType) event->state; }
	if (button != mouse_tool::None) current_tool->MotionEvent(GetWindow(widget), x, y);
	return TRUE;
}

void gtkgl_window::SignalHandler(GtkWidget * widget, int signal)
{
	GetWindow(widget)->SignalEvent(signal);
}

void gtkgl_window::PopupHandler(GtkWidget *, int signal)
{
	gtkgl_window * win = GetWindow(latest_popup_owner);
	bs_gui * gui = win->gui;
	
	char input;		// temporary...
	light * new_light;	// temporary...
	
	switch (signal)
	{
		case SIGNAL_POPUP_VIEWS_CREATE_CAMERA:
		gui->AddCamera(win->cam->focus);
		break;
		
		case SIGNAL_POPUP_VIEWS_CREATE_VIEW:
		gui->AddWindow(win->cam);
		break;
		
		case SIGNAL_POPUP_VIEWS_DELETE_VIEW:
//		gui->RemoveWindow(win);				// SOME PROBLEMS HERE....
		break;
		
		case SIGNAL_POPUP_LIGHTS_CREATE:
		cout << "add light:" << endl;
		cout << "directional- or spotlight ? (d/s) "; cin >> input;
		
		if (input == 'd') new_light = new directional_light(ol_static());
		else if (input == 's')
		{
			new_light = new spot_light(ol_static());
			
			const fGL trans[3] = { 0.0, 0.0, -1.0 };
			new_light->TranslateObject((const fGL *) trans, win->cam->GetLocData());
		}
		else break;
		
		cout << "global or local light ? (g/l) "; cin >> input;
		
		if (input == 'g') gui->AddGlobalLight(new_light);
		else if (input == 'l') gui->AddLocalLight(new_light, win->cam);
		else delete new_light;
		
		if (input == 'g' || draw_info) gui->UpdateAllWindows();
		else if (input == 'l') gui->UpdateWindows(win->cam);
		break;
		
		case SIGNAL_POPUP_LIGHTS_SWITCH_LOCAL:
		win->cam->use_local_lights = !win->cam->use_local_lights;
		cout << "local lights = " << (win->cam->use_local_lights ? "on" : "off") << endl;
		gui->SetupLights(win->cam); gui->UpdateWindows(win->cam);
		break;
		
		case SIGNAL_POPUP_LIGHTS_SWITCH_GLOBAL:
		win->cam->use_global_lights = !win->cam->use_global_lights;
		cout << "global lights = " << (win->cam->use_global_lights ? "on" : "off") << endl;
		gui->SetupLights(win->cam); gui->UpdateWindows(win->cam);
		break;
		
		default:
		gui->PopupEvent(win, signal);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

gtkgl_factory * gtkgl_factory::instance = NULL;
singleton_cleaner<gtkgl_factory> gtkgl_factory_cleaner;

gtkgl_factory::gtkgl_factory() : window_factory()
{
}

gtkgl_factory::~gtkgl_factory()
{
}

gtkgl_factory * gtkgl_factory::GetInstance(void)
{
	if (instance != NULL) return instance;
	else return (instance = new gtkgl_factory());
}

window * gtkgl_factory::Produce(bs_gui * mdl, camera * cam)
{
	return new gtkgl_window(mdl, cam);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET3

// eof
