// T3WINDOW.H : a view window class for GTK+.

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef T3WINDOW_H
#define T3WINDOW_H

#ifdef TARGET3

class gtkgl_window;
class gtkgl_factory;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>

#include "t3main.h"
#include "window.h"

#include BEGIN_INCLUDE_STL	// this is compiler-dependent...
#include <vector>
#include END_INCLUDE_STL	// this is compiler-dependent...

////////////////////////////////////////////////////////////////////////////////////////////////////

#define SIGNAL_SELECTRENDER		0x5011
#define SIGNAL_SELECTALL		0x5012
#define SIGNAL_INVERTSELECTION		0x5013
#define SIGNAL_DELETESELECTED		0x5014

#define SIGNAL_NEWCAMERA		0x5020
#define SIGNAL_NEWWINDOW		0x5021
#define SIGNAL_REMOVEWINDOW		0x5022
#define SIGNAL_SWITCHLIGHTS		0x5023
#define SIGNAL_SELECTLIGHT		0x5024
#define SIGNAL_NEWLIGHT			0x5025

#define SIGNAL_SWITCHFOG			0x5026
#define SIGNAL_SWITCHACCUMULATION		0x5027
#define SIGNAL_SWITCHPERSPECTIVE		0x5028
#define SIGNAL_SWITCHINFOLINE			0x5029
#define SIGNAL_SWITCHQUICKUPDATE		0x5030

#define SIGNAL_EXAMPLEDIALOGOPEN		0x5050
#define SIGNAL_EXAMPLEDIALOGPROCESS		0x5051

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_window : public window
{
	private:
	
	GtkWidget * gtkwin;
	GtkWidget * glarea;
	
	static vector<gtkgl_window *> gtkgl_vector;
	static GtkWidget * latest_popup_owner;
	
	public:
	
	gtkgl_window(bs_gui *, camera *);
	~gtkgl_window();
	
	void SetCurrent(void);		// virtual
	void SetTitle(const char *);		// virtual
	
	void Update(bool);			// virtual
	
	private:
	
	void ExposeEvent(void);
	void SignalEvent(int);
	
	static gtkgl_window * GetWindow(GtkWidget *);
	
	static gint ExposeHandler(GtkWidget *, GdkEventExpose *);
	static void SignalHandler(GtkWidget *, int);	// is this ok ?!?!?!!?
	
	static gint ButtonHandler(GtkWidget *, GdkEventButton *);
	static gint MotionNotifyHandler(GtkWidget *, GdkEventMotion *);
	static gint ConfigureHandler(GtkWidget *, GdkEventConfigure *);
	static void DestroyHandler(GtkObject *);
	
	public:
	
	static void PopupHandler(GtkWidget *, int);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_factory : public window_factory
{
	private:
	
	static gtkgl_factory * instance;
	gtkgl_factory(void);
	
	public:
	
	~gtkgl_factory(void);
	static gtkgl_factory * GetInstance(void);
	
	window * Produce(bs_gui *, camera *);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET3
#endif  // T3WINDOW_H

// eof
