// WINDOW.CPP

// Copyright (C) 1998 Tommi Hassinen, Jarno Huuskonen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef TARGET1
#include "window.h"

#include "viewing.h"
#include "guibase.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

mouse_tool::state window::state = mouse_tool::Unknown;
mouse_tool::button window::button = mouse_tool::None;

bool window::shift_down = false;
bool window::ctrl_down = false;

draw_tool window::tool_draw;
select_tool window::tool_select;

translate_xy_tool window::tool_translate_xy;
translate_z_tool window::tool_translate_z;

orbit_xy_tool window::tool_orbit_xy;
orbit_z_tool window::tool_orbit_z;

rotate_xy_tool window::tool_rotate_xy;
rotate_z_tool window::tool_rotate_z;

zoom_tool window::tool_zoom;
control_tool window::tool_control;

mouse_tool * window::current_tool = & window::tool_orbit_xy;	// tool_orbit_xy is the default tool...

bool window::quick_update = false;
bool window::draw_info = true;

////////////////////////////////////////////////////////////////////////////////////////////////////

window::window(bs_gui * p1, camera * p2)
{
	gui = p1; cam = p2;
	
//	render = RENDER_WIREFRAME;		// make this model-dependent like the coloring...
	render = RENDER_BALL_AND_STICK;		// make this model-dependent like the coloring...
	label = LABEL_NOTHING;
	
	colormode = p1->GetDefaultColorMode();
	
	enable_fog = false;
	accumulate = false;
}

window::~window(void)
{
}

void window::Render(void)
{
	SetCurrent();
	cam->RenderWindow(this);
}

void window::SetSize(i32s p1, i32s p2)
{
	size[0] = p1; size[1] = p2;
	if (!size[0] || !size[1]) size[0] = NOT_DEFINED;
	else glViewport(0, 0, size[0], size[1]);
	Update(false);
}

void window::GetCRD(i32s * p1, fGL * p2)
{
	v3d<fGL> xdir = (cam->GetLocData()->ydir).vpr(cam->GetLocData()->zdir); xdir = xdir / xdir.len();
	v3d<fGL> tmpv = v3d<fGL>(cam->GetLocData()->crd); tmpv = tmpv + (cam->GetLocData()->zdir * cam->focus);
	tmpv = tmpv + xdir * (2.0 * range[0] * (fGL) (size[0] / 2 - p1[0]) / (fGL) size[0]);
	tmpv = tmpv + cam->GetLocData()->ydir * (2.0 * range[1] * (fGL) (size[1] / 2 - p1[1]) / (fGL) size[1]);
	for (i32s n1 = 0;n1 < 3;n1++) p2[n1] = tmpv[n1];
}

void window::WriteBitmapString(void * font, const char * str)
{
	i32u count = 0;
	while (count < strlen(str))
	{
		glutBitmapCharacter(font, str[count++]);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET1

// eof
