// WINDOW.H : a view window base class, easy to use on different platforms...

// Copyright (C) 1998 Tommi Hassinen, Jarno Huuskonen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef WINDOW_H
#define WINDOW_H

#ifndef TARGET1

class window;
class window_factory;

////////////////////////////////////////////////////////////////////////////////////////////////////

class camera;		// viewing.h

class color_mode;	// model.h
class bs_gui;		// model.h

#include "mtools.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

// "window" is a base class for all viewing windows. all operating system/windowing environment-
// dependent things are wrapped in ctor/dtor or a set of pure virtual functions. some mouse tool-
// related and other utility stuff is also here...

class window	// RENAME TO "VIEW_WINDOW" FOR CLARITY !??!?!
{
	public:
	
	bs_gui * gui;
	camera * cam;
	
	i32s size[2];		// size of the window, in pixels.
	fGL range[2];		// dimensions of the view, in nanometers.
	
	i32s render; i32s label;
	color_mode * colormode;
	
	bool enable_fog;
	bool accumulate;
	
	// static data about mouse/keyboard state...
	
	static mouse_tool::state state;
	static mouse_tool::button button;
	
	static bool shift_down;
	static bool ctrl_down;
	
	// all available mouse tools are here...
	
	static draw_tool tool_draw;
	static select_tool tool_select;
	
	static translate_xy_tool tool_translate_xy;
	static translate_z_tool tool_translate_z;
	
	static orbit_xy_tool tool_orbit_xy;
	static orbit_z_tool tool_orbit_z;
	
	static rotate_xy_tool tool_rotate_xy;
	static rotate_z_tool tool_rotate_z;
	
	static zoom_tool tool_zoom;
	static control_tool tool_control;
	
	// pointer to the currently selected mouse tool.
	
	static mouse_tool * current_tool;
	
	// some other static flags...
	
	static bool quick_update;
	static bool draw_info;
	
	public:
	
	window(bs_gui *, camera *);
	virtual ~window(void);
	
	void Render(void);
	void SetSize(i32s, i32s);
	void GetCRD(i32s *, fGL *);
	
	void WriteBitmapString(void *, const char *);
	
	virtual void SetCurrent(void) = 0;
	virtual void SetTitle(const char *) = 0;
	
	virtual void Update(bool) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

// "window_factory" is a platform-independent way to create viewing windows. for each different
// platform (just like a "window") a "window_factory" should be made which produces a new "window"
// for this particular platform...

class window_factory
{
	protected:
	
	public:
	
	window_factory(void) { };
	virtual ~window_factory(void) { };
	
	virtual window * Produce(bs_gui *, camera *) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// TARGET1
#endif	// WINDOW_H

// eof
